;;; texi-caption.el --- -*- emacs-lisp -*- enhancements for editing texi-caption

;;; Copyright (C) 2001, 2003 Gary V. Vaughan <gary@gnu.org>
;;; Renamed as caption.el and modified by Akim Demaille
;;; <demaille@freefriends.org>

;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2 of the license, or
;; (at your option) any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, write to the Free Software
;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


;;; Commentary:

;; Before allowing the buffer local variables in texi-caption texinfo sources
;; to trigger, you need to have this file in your emacs load path, by
;; adding something like the followinf to your .emacs file:
;;
;;        (setq load-path
;;           (append load-path (expand-file-name "~/texi-caption")))
;;
;; After that, the files take care of themselves by hooking into the code
;; below.

;; Code:

(require 'cl)
(provide 'texi-caption)

(defvar texi-caption-chapno 1
  "*The chapter number represented by the current buffer.  Set this in the
   Local variables section of each source file.")
(make-variable-buffer-local 'texi-caption-chapno)
(set-default 'texi-caption-chapno nil)

(defvar texi-caption-captno 1
  "*The ordinal number of the first caption in the current buffer.  If a
   chapter is split over several files, you might need to set this in the
   Local variables section of those files.")
(make-variable-buffer-local 'texi-caption-captno)
(set-default 'texi-caption-captno 1)

(defvar texi-caption-caption-update-p nil
  "*Whether the current buffer requires caption number updates.")
(make-variable-buffer-local 'texi-caption-caption-update-p)
(set-default 'texi-caption-caption-update-p nil)

(defvar texi-caption-caption-regexp "@[a-z][a-z]*caption{"
  "Regular expression to match the text just before each updatable caption
number in a file.")

(defvar texi-caption-exref-regexp "@exref{"
  "Regular expression to match the text just before each updatable caption
reference in a file.")

(defvar texi-caption-caption-text-regexp "[{}]"
  "Regular expression to match the caption text in an excaption macro")




(defun texi-caption-buffer-caption ()
  "Evaluates to the text between point and the next unmatched `}'"

  (while (looking-at "[ \n,]")
    (forward-char 1))

  (save-excursion
    (let ((nesting 1)
	  (beg (point)))

      (while (> nesting 0)
	(re-search-forward texi-caption-caption-text-regexp nil t)
	(goto-char (match-beginning 0))
	(cond
	 ((looking-at "{") (setq nesting (1+ nesting)))
	 ((looking-at "}") (setq nesting (1- nesting))))
	(goto-char (match-end 0)))

      (replace-regexp-in-string "\n" " " (buffer-substring beg (1- (point)))))))



(defun texi-caption-update-caption-numbers ()
  "Update each of the caption numbers in the current buffer."
  (if texi-caption-caption-update-p
      (let ((caption texi-caption-captno)
	    (capname-alist nil))
	(save-excursion

	  ;;; --------------------------------------- ;;;
	  ;;; FIRST PASS -- collect @excaption macros ;;;
	  ;;; --------------------------------------- ;;;

	  ;; start at the beginning of the buffer
	  (goto-char (point-min))

	  ;; for each @excaption invocation found...
	  (while (re-search-forward texi-caption-caption-regexp nil t)
	    (let ((capentry (cons nil
				  (if texi-caption-chapno
				      (format "%d.%d" texi-caption-chapno caption)
				    (format "%d" caption)))))
	      (goto-char (match-end 0))

	      ;; delete the old caption number
	      (while (not (looking-at ","))
		(delete-char 1))

	      ;; and replace it with a new sequential number
	      (insert (cdr capentry))

	      ;; capture the caption text
	      (setcar capentry (texi-caption-buffer-caption))

	      (setq capname-alist
		    (append capname-alist (list capentry))
		    caption (1+ caption))))

	  ;;; ---------------------------------------- ;;;
	  ;;; SECOND PASS -- correct @exref references ;;;
	  ;;; ---------------------------------------- ;;;

	  (goto-char (point-min))

	  (while (re-search-forward texi-caption-exref-regexp nil t)
	    (let ((capref "")
		  (beg (point)))

	      ;; find the start of the cation reference
	      (while (not (looking-at ","))
		(forward-char 1))
	      (setq capref (texi-caption-buffer-caption))
	      (goto-char beg)

	      ;; delete the old caption number
	      (while (not (looking-at ","))
		(delete-char 1))

	      ;; and replace it with a new matching number
	      (let ((res (cdr (assoc capref capname-alist))))
		(if res
		    (insert res)
		  (error "unknown caption: %s" capref)))))

	  ))))

;;; texi-caption.el ends here
